// BaseClass.h
//
// ICS 65 Fall 2012
// Code Example
//
// This class is an abstract base class for a couple of derived classes.
// It doesn't do anything useful, except that it has a virtual clone()
// function (which is pure virtual -- abstract -- in this class, but will
// be implemented in the derived classes).

#ifndef BASECLASS_H
#define BASECLASS_H

#include <string>
#include <iostream>



class BaseClass
{
public:
	BaseClass(int i, const std::string& s): ii(i), ss(s) { }
	virtual ~BaseClass() { }

	int getII() const { return ii; }
	std::string getSS() const { return ss; }

	// This is the virtual "clone" function.  Its job is to create a
	// dynamically-allocated copy of this object and return a pointer
	// to it.  The magic, though, is that it's a virtual function, so
	// you can call it on a BaseClass* pointing to an object of any
	// derived class of BaseClass, and the appropriate version of the
	// function will be called.  Each different version of the function
	// will create a new object of a different type.  (I've marked the
	// function "const" because you can copy an object without changing
	// it.)
	//
	// The reason why this function is pure virtual is because you can't
	// create BaseClass objects, since BaseClass is abstract.
	virtual BaseClass* clone() const = 0;


	// This is a virtual "print" function.  I've implemented this as a
	// function, rather than as an operator, because operators are not
	// polymorphic.
	virtual void print(std::ostream& out) const;


private:
	int ii;
	std::string ss;
};



#endif // BASECLASS_H
