// DerivedClass1.h
//
// ICS 65 Fall 2012
// Code Example
//
// This class is derived from BaseClass and has its own implementation of
// the clone() function; in this case, the clone() function returns an
// object of the appropriate type (DerivedClass1), which is a copy of
// "this" object.

#include "BaseClass.h"



class DerivedClass1 : public BaseClass
{
public:
	DerivedClass1(int i, const std::string& s, bool b): BaseClass(i, s), bb(b) { }
	virtual ~DerivedClass1() { }


	bool getBB() const { return bb; }


	// Notice that this clone() function has a different signature than
	// the clone() function in BaseClass: in particular, it returns
	// DerivedClass1*, rather than BaseClass*.  This function is, nonetheless,
	// considered to be an override of the BaseClass::clone() function.  The
	// reason is that the only difference is in the return type, and the
	// difference is a movement "down" the inheritance hierarchy.  This kind
	// of change is type-safe, so it's allowed in C++; the feature is known
	// as "covariant return types."
	//
	// The reason why the change in return type is desirable is so that you
	// can copy a DerivedClass1 object through a DerivedClass1* and get back
	// a DerivedClass1*.  (If you only got back a BaseClass*, you might have
	// to do a static_cast or dynamic_cast on the resulting pointer in order
	// to use it.)  Java added this feature in its version 5.0, but most of
	// the older code in the Java library is built without using this feature;
	// so, in Java, every time you clone an object, you get back an Object
	// reference, which you then have to downcast in order to do anything
	// useful with it.
	virtual DerivedClass1* clone() const
	{
		// Return a copy of "this" DerivedClass1 object.  It's safe to do this,
		// even though I haven't provided a copy constructor, since the
		// default copy constructor will do fine.
		return new DerivedClass1(*this);
	}


	virtual void print(std::ostream& out) const;


private:
	bool bb;
};
