// main.cpp
//
// ICS 65 Fall 2012
// Code Example
//
// Here is an example of cloning and printing objects of our derived
// classes polymorphically.

#include "BaseClass.h"
#include "DerivedClass1.h"
#include "DerivedClass2.h"
#include <iostream>
#include <vector>
using namespace std;


int main()
{
	// This vector will contain a mixture of objects of different types
	// (some will be DerivedClass1 objects, some DerivedClass2).
	vector<BaseClass*> v;
	v.push_back(new DerivedClass1(3, "Alex", true));
	v.push_back(new DerivedClass2(8, "Happy", 3.5));
	v.push_back(new DerivedClass1(4, "Boo", false));
	v.push_back(new DerivedClass1(9, "Hello", true));
	v.push_back(new DerivedClass2(10, "Goodbye", -1.0));

	// If we loop through the vector and print all of the objects by calling
	// their virtual print() function, the "right thing" will happen, because
	// print() is virtual.
	for (BaseClass* b : v)
	{
		b->print(cout);
		cout << endl;
	}

	// Now suppose we want to make a new vector that contains pointers to
	// copies of the objects in v.  This clearly won't do the job:
	//
	//     vector<BaseClass*> v2 = v;
	//
	// because it will copy the pointers from v, but not the objects those
	// pointers point to.  In order to copy the actual objects, we'll need
	// to dynamically allocate new copies.  Dynamic allocation requires us
	// to use "new", but it also requires us to know what kind of object to
	// create.  But given a BaseClass*, we don't know, in general, what kinds
	// of objects they point to.  Some will be DerivedClass1s, some will be
	// DerivedClass2s, and we don't know what that mixture will be.
	//
	// But we have an ace in the hole: BaseClass has a virtual function called
	// clone() that knows how to dynamically allocate a copy of "this" that
	// is the right type.  Depending on what kind of object we call clone()
	// on, the "right version" of clone() is called (because it's virtual),
	// which will know what kind of object to create, just as our calls to
	// print() could do the "right thing" automatically.  It's the same
	// concept: polymorphism to the rescue!

	vector<BaseClass*> v2;

	for (BaseClass* b : v)
	{
		// Here, we call clone() to make a copy of each object in v.
		// Depending on what kind of object it is, the right type of
		// object will be created.
		v2.push_back(b->clone());
	}

	// Let's verify the copies are correct, by printing them to the console.
	for (BaseClass* b : v2)
	{
		b->print(cout);
		cout << endl;
	}

	// Now we can delete the objects in each vector.  The destructors are
	// virtual, so the right destructors are being called, too!
	for (BaseClass* b : v)
	{
		delete b;
	}

	for (BaseClass* b : v2)
	{
		delete b;
	}

	return 0;
}

